@@ -1153,7 +1153,7 @@ FindPiece(src, position, first)
 AsciiSrcObject src;
 XawTextPosition position, *first;
 {
-  Piece * old_piece, * piece = src->ascii_src.first_piece;
+  Piece *piece = src->ascii_src.first_piece, *old_piece = piece;
   XawTextPosition temp;
 
   for ( temp = 0 ; piece != NULL ; temp += piece->used, piece = piece->next ) {
