@@ -134,7 +134,7 @@ WidgetClass boxWidgetClass = (WidgetClass)&boxClassRec;
  *
  */
 
-static DoLayout(bbw, width, height, reply_width, reply_height, position)
+static void DoLayout(bbw, width, height, reply_width, reply_height, position)
     BoxWidget	bbw;
     Dimension	width, height;
     Dimension	*reply_width, *reply_height; /* bounding box */
@@ -208,8 +208,7 @@ static DoLayout(bbw, width, height, reply_width, reply_height, position)
 
     if (!vbox && width && lw > width && lh < height) {
 	/* reduce width if too wide and height not filled */
-	Dimension sw = lw, sh = lh;
-	Dimension width_needed;
+	Dimension width_needed = lw, sw = lw, sh = lh;
 	XtOrientation orientation = bbw->box.orientation;
 	bbw->box.orientation = XtorientVertical;
 	while (sh < height && sw > width) {
@@ -402,6 +401,7 @@ static Boolean TryNewLayout(bbw)
 				     &proposed_width, &proposed_height))
 	{
 	    case XtGeometryYes:
+	    case XtGeometryDone:
 		return (TRUE);
 
 	    case XtGeometryNo:
