@@ -23,9 +23,10 @@ ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
 SOFTWARE.
 
 3D extensions added and extensively redesigned: Roland Krause 1996
-Helper function added: Roland Krause 1998
+Tooltip function added: Roland Krause 1998
 Redesign for KDE styled buttons: Roland Krause 1999
 Adaption to the new Add3dExt concept: Roland Krause 1999
+New mechanism to truncate long label strings : Roland Krause 1999
 
 ******************************************************************/
 
@@ -296,6 +297,7 @@ CommandWidget cw;
   cw->command.helpLabel = XtVaCreateManagedWidget("helpLabel",
 				labelWidgetClass, cw->command.helpWidget,
 				XtNencoding, cw->command.helpEncoding,
+				XtNtruncLabel, False,
 				XtNlabel, cw->command.helpText,
 				XtNbackground, cw->command.helpBackground, NULL);
 }
@@ -359,7 +361,7 @@ Widget request, new;
 ArgList args;			/* unused */
 Cardinal *num_args;		/* unused */
 {
-  Dimension Add;
+  Dimension Add, OldWidth;
   CommandWidget cbw = (CommandWidget)new;
 
   cbw->command.set = FALSE;
@@ -367,10 +369,13 @@ Cardinal *num_args;		/* unused */
   /* Calculate the correct size of the widget */
 
   Add = 2 * (cbw->add3dExt.borderWidth + cbw->command.highlight_thickness);
+  OldWidth = cbw->core.width;
   cbw->core.width  += Add;
   cbw->core.height += Add;
   RepositionLabel(new, new);
-
+  if (cbw->label.truncLabel)
+    TruncateLabelString((LabelWidget)new, OldWidth - LEFT_OFFSET((LabelWidget)new));
+    
   /* Initialize the help widget, if required */
 
   cbw->command.helpTimer = 0;
@@ -423,6 +428,9 @@ Widget w;
   unsigned int height= w->core.height - 2*offset;
 
   RepositionLabel(w, w);
+  if (cbw->label.truncLabel)
+    TruncateLabelString((LabelWidget)w, width - 2*cbw->add3dExt.borderWidth -
+    			 LEFT_OFFSET(cbw));
 
   if (XtIsRealized(w) && (cbw->command.highlightMode == FALSE)) 
     XawRaisedRectangle(w, offset, offset, width, height);
@@ -502,9 +510,14 @@ Cardinal *num_args;
     cbw->core.height = CalcHeight(cbw);
     DoResize = TRUE;
   }
-  if (DoResize)
+  if (DoResize || (oldcbw->label.label != cbw->label.label))
   {
     RepositionLabel(current, new);
+    if (cbw->label.truncLabel)
+      TruncateLabelString((LabelWidget)new, oldcbw->core.width -
+        2*(cbw->add3dExt.borderWidth + cbw->command.highlight_thickness) -
+	LEFT_OFFSET(cbw));
+
     redisplay = TRUE;
   }
 
