@@ -23,6 +23,8 @@ ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
 SOFTWARE.
 
 3D extensions and XPM support added : Roland Krause 1996, 1997
+Changes for the new XPM conversion mechanism : Roland Krause 1999
+New mechanism to truncate long label strings : Roland Krause 1999
 
 ******************************************************************/
 
@@ -36,6 +38,7 @@ SOFTWARE.
 #include <X11/Xos.h>
 #include <X11/XawPlus/XawInit.h>
 #include <X11/XawPlus/LabelP.h>
+#include <X11/XawPlus/StrToPmap.h>
 #include <X11/Xmu/Converters.h>
 #include <X11/Xmu/Drawing.h>
 #include <stdio.h>
@@ -48,8 +51,6 @@ SOFTWARE.
 #define WORD64
 #endif
 
-extern void XmuCvtStringToPixmap();	/* The XPM converter */
-
 /****************************************************************
  *
  * Full class record constant
@@ -74,14 +75,24 @@ static XtResource resources[] = {
 	offset(label.internal_width), XtRImmediate, (XtPointer)4},
     {XtNinternalHeight, XtCHeight, XtRDimension, sizeof(Dimension),
 	offset(label.internal_height), XtRImmediate, (XtPointer)2},
+ 
     {XtNresize, XtCResize, XtRBoolean, sizeof(Boolean),
 	offset(label.resize), XtRImmediate, (XtPointer)True},
 
+    {XtNtruncLabel, XtCTruncate, XtRBoolean, sizeof(Boolean),
+	offset(label.truncLabel), XtRImmediate, (XtPointer)True},
+    {XtNtruncLeft, XtCTruncate, XtRBoolean, sizeof(Boolean),
+	offset(label.truncLeftSide), XtRImmediate, (XtPointer)False},
+
     {XtNbitmap, XtCPixmap, XtRBitmap, sizeof(Pixmap),
 	offset(label.pixmap), XtRImmediate, (XtPointer)None},
+    {XtNclipMask, XtCPixmap, XtRClipMask, sizeof(Pixmap),
+	offset(label.clipMask), XtRImmediate, (XtPointer)None},
 
     {XtNleftBitmap, XtCLeftBitmap, XtRBitmap, sizeof(Pixmap),
-       offset(label.left_bitmap), XtRImmediate, (XtPointer)None}
+       offset(label.left_bitmap), XtRImmediate, (XtPointer)None},
+    {XtNleftClipMask, XtCLeftBitmap, XtRClipMask, sizeof(Pixmap),
+       offset(label.left_clipMask), XtRImmediate, (XtPointer)None},
 };
 #undef offset
 
@@ -95,12 +106,15 @@ static void ClassInitialize();
 static void Destroy();
 static XtGeometryResult QueryGeometry();
 
+#define superclass	(&simpleClassRec)
+#define ownClassName	"Label"
+
 LabelClassRec labelClassRec = {
   {
 /* core_class fields */	
-#define superclass		(&simpleClassRec)
+
     /* superclass	  	*/	(WidgetClass) superclass,
-    /* class_name	  	*/	"Label",
+    /* class_name	  	*/	ownClassName,
     /* widget_size	  	*/	sizeof(LabelRec),
     /* class_initialize   	*/	ClassInitialize,
     /* class_part_initialize	*/	NULL,
@@ -339,7 +353,6 @@ LabelWidget clw, lw;
 	      &lw->label.lbm_width, &lw->label.lbm_height, &bw, &lw->label.label_depth))) {
 	lw->label.lbm_width = lw->label.lbm_height = 0;
     }
-    compute_bitmap_offsets(clw, lw);
 }
 
 /***********************************************************************
@@ -360,12 +373,15 @@ static void ClassInitialize()
   static XtConvertArgRec xpmCvtArg[] = {
     {XtWidgetBaseOffset,(XtPointer)XtOffsetOf(LabelRec,core.screen),sizeof(Screen *)},
     {XtWidgetBaseOffset,(XtPointer)XtOffsetOf(LabelRec,core.colormap),sizeof(Colormap)},
-    {XtWidgetBaseOffset,(XtPointer)XtOffsetOf(LabelRec,label.xpm_clipmask),sizeof(Pixmap *)}
   };
 
   XawInitializeWidgetSet();
-  XtAddConverter(XtRString,XtRJustify,XmuCvtStringToJustify, NULL, 0);
-  XtAddConverter(XtRString,XtRBitmap,XmuCvtStringToPixmap,xpmCvtArg,XtNumber(xpmCvtArg));
+  XtAddConverter(XtRString, XtRJustify, XmuCvtStringToJustify, NULL, 0);
+
+  XtAddConverter(XtRString, XtRBitmap, XawCvtStringToPixmap,
+		     xpmCvtArg, XtNumber(xpmCvtArg));
+  XtAddConverter(XtRString, XtRClipMask, XawCvtStringToClipMask,
+		     xpmCvtArg, XtNumber(xpmCvtArg));
 }
 
 /***********************************************************************
@@ -389,6 +405,7 @@ static void Initialize(request, new)
     GetgrayGCs(lw);
 
     SetTextWidthAndHeight(lw);
+    lw->label.label_full_len = NULL;
 
     if (lw->core.height == 0)
         lw->core.height = lw->label.label_height + 2*lw->label.internal_height;
@@ -449,51 +466,49 @@ Widget w;
 XEvent *event;
 Region region;
 {
-   LabelWidget lw = (LabelWidget) w;
-   GC	       gc = lw->label.normal_GC;
-
-   if (region != NULL) {
-       int x = lw->label.label_x;
-       unsigned int width = lw->label.label_width;
-       if (lw->label.lbm_width) {
-	   if (lw->label.label_x > (x = lw->label.internal_width))
-	       width += lw->label.label_x - x;
-       }
-       if (XRectInRegion(region, x, lw->label.label_y,
-			 width, lw->label.label_height) == RectangleOut)
-	   return;
-   }
-
-   if (lw->label.pixmap == None) {
-       int len = lw->label.label_len;
-       char *label = lw->label.label;
-       Position y = lw->label.label_y + lw->label.font->max_bounds.ascent;
+   LabelWidget	lw = (LabelWidget) w;
+   GC		gc = lw->label.normal_GC;
+   int		len = lw->label.label_len;
+   char		*label = lw->label.label;
+   Position	y = lw->label.label_y + lw->label.font->max_bounds.ascent;
 
+   if (lw->label.pixmap == None)
+   {
        /* Display left bitmap or pixmap */
 
-       if (lw->label.left_bitmap && lw->label.lbm_width != 0) {
-	 if (lw->label.label_depth == 1)
-	   XCopyPlane(XtDisplay(w), lw->label.left_bitmap, XtWindow(w), gc,
-		       0, 0, lw->label.lbm_width, lw->label.lbm_height,
-		       (int) lw->label.lbm_x, 
-		       (int) lw->label.internal_height + lw->label.lbm_y,
-		       (unsigned long) 1L);
-         else {
-	   if (lw->label.xpm_clipmask != None) {
-	     XSetClipMask(XtDisplay(w), gc, lw->label.xpm_clipmask);
-	     XSetClipOrigin(XtDisplay(w), gc, lw->label.lbm_x,
-			    lw->label.internal_height + lw->label.lbm_y);
-	   }
-	   XCopyArea(XtDisplay(w), lw->label.left_bitmap, XtWindow(w), gc,
+       if (lw->label.left_bitmap && lw->label.lbm_width != 0)
+       {
+	 /* We only display the left bitmap/pixmap if it is in the region to redisplay. */
+
+	 if ((event == NULL) || (XRectInRegion(region, lw->label.lbm_x, lw->label.lbm_y,
+	      lw->label.lbm_width, lw->label.lbm_height) != RectangleOut))
+	 {
+	   if (lw->label.label_depth == 1)
+	     XCopyPlane(XtDisplay(w), lw->label.left_bitmap, XtWindow(w), gc,
+		        0, 0, lw->label.lbm_width, lw->label.lbm_height,
+		        (int) lw->label.lbm_x, 
+		        (int) lw->label.internal_height + lw->label.lbm_y,
+		        (unsigned long) 1L);
+           else {
+	     if (lw->label.left_clipMask != None) {
+	       XSetClipMask(XtDisplay(w), gc, lw->label.left_clipMask);
+	       XSetClipOrigin(XtDisplay(w), gc, lw->label.lbm_x,
+			      lw->label.internal_height + lw->label.lbm_y);
+	     }
+	     XCopyArea(XtDisplay(w), lw->label.left_bitmap, XtWindow(w), gc,
 		       0, 0, lw->label.lbm_width, lw->label.lbm_height,
 		       (int) lw->label.lbm_x, 
 		       (int) lw->label.internal_height + lw->label.lbm_y);
-	   XSetClipMask(XtDisplay(w), gc, None);
-         }
+	     XSetClipMask(XtDisplay(w), gc, None);
+           }
+	 }
        }
-       /* Display label text */
+       /* Display label text if it is in the region to redisplay */
 
-       if (len == MULTI_LINE_LABEL) {
+       if ((event == NULL) || (XRectInRegion(region, lw->label.label_x, lw->label.label_y,
+	    lw->label.label_width, lw->label.label_height) != RectangleOut))
+       {
+         if (len == MULTI_LINE_LABEL) {
 	   char *nl;
 	   while ((nl = index(label, '\n')) != NULL)
 	   {
@@ -503,8 +518,9 @@ Region region;
 	     label = nl + 1;
 	   }
 	   len = strlen(label);
+         }
+         if (len) _DrawLabelString(lw, label, (int)y, len);
        }
-       if (len) _DrawLabelString(lw, label, (int)y, len);
    }
    else	/* Label with only a bitmap or pixmap */
    {
@@ -513,8 +529,8 @@ Region region;
 		   0, 0, lw->label.label_width, lw->label.label_height,
 		   lw->label.label_x, lw->label.label_y, 1L);
       else {
-	if (lw->label.xpm_clipmask != None) {
-	  XSetClipMask(XtDisplay(w), gc, lw->label.xpm_clipmask);
+	if (lw->label.clipMask != None) {
+	  XSetClipMask(XtDisplay(w), gc, lw->label.clipMask);
 	  XSetClipOrigin(XtDisplay(w), gc, lw->label.label_x, lw->label.label_y);
 			    
 	}
@@ -526,13 +542,132 @@ Region region;
    }
 }
 
+/***********************************************************************
+ *
+ * TruncateLabelString: Truncate the label string, if it is too 
+ *			long for the widgets window.
+ *
+ ***********************************************************************/
+
+/* DoTruncate(): Truncate the label string.
+ * This function does not free the memory used by the label string.
+ */
+
+static void DoTruncate8(lw, usableWidth)
+LabelWidget lw;
+int usableWidth;
+{
+   int newlen, fullLen, currentWidth;
+   static char *dots = "..";
+   char *p;
+   
+   fullLen = strlen(lw->label.label_full_len);
+
+   /* Handling for a fixed width font: Calculate the string length */
+	 
+   if (lw->label.font->max_bounds.width == lw->label.font->min_bounds.width)
+     newlen = usableWidth / lw->label.font->max_bounds.width;
+   else
+   {
+     /* Handling for a proportional font:
+      * Calculate character by character how long the string have to be.
+      */
+     currentWidth = XTextWidth(lw->label.font, dots, 2);
+     newlen = 0;
+
+     if (lw->label.truncLeftSide) {	/* Truncate from the left side */
+	p = &lw->label.label_full_len[fullLen];
+        while ((currentWidth < usableWidth) && (newlen < fullLen)) {
+           newlen++; p--;
+	   currentWidth += XTextWidth(lw->label.font, p, 1);
+        }
+        newlen++;	/* for the dots, otherwise we have to decrement newlen */
+     }
+     else {				/* Truncate from the right side */
+	p = lw->label.label_full_len;
+        while ((currentWidth < usableWidth) && (newlen < fullLen)) {
+           newlen++;
+	   currentWidth += XTextWidth(lw->label.font, p, 1);
+	   p++;
+        }
+        newlen++;	/* ... for the dots */
+     }
+   }
+   /* Check if our truncate mechanism works with the
+    * given label width. Otherwise we only copy two dots.
+    */
+   if (newlen > 3) {
+      lw->label.label = XtMalloc(newlen+1);
+      if (lw->label.truncLeftSide) {
+         strcpy(lw->label.label, dots);
+         strcat(lw->label.label, &lw->label.label_full_len[fullLen - newlen + 2]);
+      }
+      else {
+         strncpy(lw->label.label, lw->label.label_full_len, newlen - 2);
+	 lw->label.label[newlen - 2] = '\0';
+	 strcat(lw->label.label, dots);
+      }
+      lw->label.label_len = newlen;
+   }
+   else		/* The label is too small for the truncate mechanism */
+   {
+      lw->label.label = XtMalloc(3);
+      strcpy(lw->label.label, dots);
+   }
+}
+
+/* Check if the label string have to be truncated to display
+ * it in the label. We don't do this, if we have a bitmap or
+ * a multiline label.
+ * TruncateLabelString() will modify the label and the label_len
+ * entries in the widget record.
+ */
+void TruncateLabelString(lw, currentWidth)
+LabelWidget lw;
+Dimension currentWidth;
+{
+   int usableWidth;
+
+   if ((lw->label.pixmap == None) && (lw->label.label_len != MULTI_LINE_LABEL) &&
+       (*lw->label.label != '\0') && (lw->label.encoding == False))
+   {
+      /* Label string will be displayed and is not multi line:
+       * Check if the string fits into the widget.
+       */
+      usableWidth = currentWidth - 2 * lw->label.internal_width;
+      if (lw->label.label_width > usableWidth)
+      {
+         /* Widget size is to small to display the current label string.
+	  * The label string will be truncated until it fits into the
+	  * widget.
+	  */
+	 if (lw->label.label_full_len == NULL) lw->label.label_full_len = lw->label.label;
+	 else XtFree(lw->label.label);
+	 DoTruncate8(lw, usableWidth);
+      }
+      else	/* After resize the label string fits into the widget */
+      {
+	 if (lw->label.label_full_len != NULL)
+	 {
+	    XtFree(lw->label.label);
+	    lw->label.label = lw->label.label_full_len;
+	    lw->label.label_full_len = NULL;
+	    lw->label.label_len = strlen(lw->label.label);
+	 }
+      }
+   }
+}
+
 /***********************************************************************
  *
  * Resize method:	Resize or reposition the label
  *
  ***********************************************************************/
 
-static void _Reposition(lw, width, height)
+/* Determine the new position of the label string
+ * and the left bitmap or a pixmap after resize.
+ */
+static void Reposition(lw, width, height)
 register LabelWidget lw;
 Dimension width, height;
 {
@@ -565,8 +700,17 @@ static void Resize(w)
 {
     LabelWidget lw = (LabelWidget)w;
 
-    _Reposition(lw, w->core.width, w->core.height);
-    compute_bitmap_offsets(lw, lw);
+    /* We only make a resize, if we are a Label widget.
+     * If Label is only the superclass, we do nothing
+     * because the derived widget class has its own
+     * resize method.
+     */
+    if (strcmp(ownClassName, XtClass(w)->core_class.class_name) == 0)
+    {
+      Reposition(lw, w->core.width, w->core.height);
+      if (lw->label.truncLabel) TruncateLabelString(lw, w->core.width - LEFT_OFFSET(lw));
+      compute_bitmap_offsets(lw, lw);
+    }
 }
 
 /***********************************************************************
@@ -575,9 +719,20 @@ static void Resize(w)
  *
  ***********************************************************************/
 
-#define PIXMAP 0
-#define WIDTH 1
-#define HEIGHT 2
+/* Free the resources allocated by the
+ * truncate mechanism
+ */
+static void ResetTruncateMode(lw)
+LabelWidget lw;
+{
+   XtFree(lw->label.label);
+   lw->label.label = lw->label.label_full_len;
+   lw->label.label_full_len = NULL;
+}
+
+#define PIXMAP	 0
+#define WIDTH	 1
+#define HEIGHT	 2
 #define NUM_CHECKS 3
 
 static Boolean SetValues(current, request, new, args, num_args)
@@ -594,7 +749,8 @@ static Boolean SetValues(current, request, new, args, num_args)
     for (i = 0; i < NUM_CHECKS; i++) checks[i] = FALSE;
 
     for (i = 0; i < *num_args; i++) {
-	if (streq(XtNbitmap, args[i].name) || streq(XtNpixmap, args[i].name))
+	if (streq(XtNbitmap, args[i].name) || streq(XtNpixmap, args[i].name) ||
+	    streq(XtNclipMask, args[i].name))
 	    checks[PIXMAP] = TRUE;
 	if (streq(XtNwidth, args[i].name))
 	    checks[WIDTH] = TRUE;
@@ -606,20 +762,24 @@ static Boolean SetValues(current, request, new, args, num_args)
     /*
      * resize on bitmap change
      */
-    if (curlw->label.left_bitmap != newlw->label.left_bitmap) was_resized = True;
+    if ((curlw->label.left_bitmap != newlw->label.left_bitmap) ||
+	(curlw->label.left_clipMask != newlw->label.left_clipMask)) was_resized = True;
     if (curlw->label.encoding != newlw->label.encoding) was_resized = True;
 
-    if (curlw->label.label != newlw->label.label) {
-        if (curlw->label.label != curlw->core.name) XtFree((char *)curlw->label.label);
-
-	if (newlw->label.label != newlw->core.name)
-          newlw->label.label = XtNewString(newlw->label.label);
-	was_resized = True;
+    if (curlw->label.label != newlw->label.label)
+    {
+       if (curlw->label.label != curlw->core.name) XtFree((char *)curlw->label.label);
+       if (newlw->label.label != newlw->core.name)
+         newlw->label.label = XtNewString(newlw->label.label);
+       if (newlw->label.label_full_len) XtFree(newlw->label.label_full_len);
+       newlw->label.label_full_len = NULL;
+       was_resized = True;
     }
 
     if (was_resized || (curlw->label.font != newlw->label.font) ||
-	(curlw->label.justify != newlw->label.justify) || checks[PIXMAP]) {
-
+	(curlw->label.justify != newlw->label.justify) || checks[PIXMAP])
+    {
+	if (newlw->label.label_full_len) ResetTruncateMode(newlw);
 	SetTextWidthAndHeight(newlw);
 	was_resized = True;
     }
@@ -649,12 +809,50 @@ static Boolean SetValues(current, request, new, args, num_args)
 	GetgrayGCs(newlw);
 	redisplay = True;
     }
+    /* Truncate machanism activated or deactivated ? */
+    
+    if (curlw->label.truncLabel != newlw->label.truncLabel)
+    {
+	/* Truncate deactivated: If label is currently truncated
+	 * release truncated string and reset the original label.
+	 */
+        if (newlw->label.truncLabel == False)
+	{
+    	  if (newlw->label.label_full_len)
+	  {
+	     ResetTruncateMode(newlw);
+	     newlw->label.label_len = strlen(newlw->label.label);
+	     redisplay = True;
+	  }
+	}
+	/* Truncate machanism activated: No resize but resize handling required */
+	
+	else was_resized = True;
+    }
+
+    if ((curlw->label.truncLeftSide != newlw->label.truncLeftSide) &&
+        (newlw->label.label_full_len != NULL))
+    {
+    	/* Mode of truncate mechanism changed and label string is truncated:
+	 * Recalculation of the string is required.
+	 */
+	ResetTruncateMode(newlw);
+	newlw->label.label_len = strlen(newlw->label.label);
+	was_resized = True;
+    }
 
     if ((curlw->label.internal_width != newlw->label.internal_width) ||
         (curlw->label.internal_height != newlw->label.internal_height) || was_resized) {
+	
 	/* Resize() will be called if geometry changes succeed */
-
-	_Reposition(newlw, curlw->core.width, curlw->core.height);
+	
+	if (strcmp(ownClassName, XtClass(new)->core_class.class_name) == 0)
+	{
+	  Reposition(newlw, curlw->core.width, curlw->core.height);
+    	  compute_bitmap_offsets(curlw, newlw);
+	  if (newlw->label.truncLabel)
+	    TruncateLabelString(newlw, curlw->core.width - LEFT_OFFSET(newlw));
+	}
     }
 
     return was_resized || redisplay ||
@@ -673,6 +871,7 @@ Widget w;
     LabelWidget lw = (LabelWidget)w;
 
     XtFree(lw->label.label);
+    if (lw->label.label_full_len != NULL) XtFree(lw->label.label_full_len);
     XtReleaseGC(w, lw->label.normal_GC);
     XtReleaseGC(w, lw->label.gray_GC);
     XtReleaseGC(w, lw->label.lgray_GC);
