@@ -446,7 +446,7 @@ GC gc;
     int hl_x, hl_y, width, height;
 
     hl_x = x - lw->list.column_space/2;
-    width  = lw->list.col_width + lw->list.column_space/2;
+    width  = lw->list.col_width;
     hl_y = y - lw->list.row_space/2;
     height = lw->list.row_height + lw->list.row_space;
 
