@@ -32,6 +32,7 @@
  *
  * 3D extensions added : Roland Krause 1997
  * XPM extensions added: Roland Krause 1997
+ * Installation of XPM type converters added: Roland Krause 1999
  */
 
 #include <X11/IntrinsicP.h>
@@ -44,6 +45,7 @@
 #include <X11/XawPlus/SimpleMenu.h>
 #include <X11/XawPlus/SmeBSBP.h>
 #include <X11/XawPlus/Cardinals.h>
+#include <X11/XawPlus/StrToPmap.h>
 
 #include <stdio.h>
 
@@ -58,13 +60,13 @@ static XtResource resources[] = {
      offset(vert_space), XtRImmediate, (XtPointer) 25},
   {XtNleftBitmap, XtCLeftBitmap, XtRBitmap, sizeof(Pixmap),
      offset(left_bitmap), XtRImmediate, (XtPointer)None},
-  {XtNleftClipMask, XtCLeftBitmap, XtRBitmap, sizeof(Pixmap),
+  {XtNleftClipMask, XtCLeftBitmap, XtRClipMask, sizeof(Pixmap),
      offset(left_clipmask), XtRImmediate, (XtPointer)None},
   {XtNjustify, XtCJustify, XtRJustify, sizeof(XtJustify),
      offset(justify), XtRImmediate, (XtPointer) XtJustifyLeft},
   {XtNrightBitmap, XtCRightBitmap, XtRBitmap, sizeof(Pixmap),
      offset(right_bitmap), XtRImmediate, (XtPointer)None},
-  {XtNrightClipMask, XtCRightBitmap, XtRBitmap, sizeof(Pixmap),
+  {XtNrightClipMask, XtCRightBitmap, XtRClipMask, sizeof(Pixmap),
      offset(right_clipmask), XtRImmediate, (XtPointer)None},
   {XtNleftMargin,  XtCHorizontalMargins, XtRDimension, sizeof(Dimension),
      offset(left_margin), XtRImmediate, (XtPointer) 4},
@@ -151,16 +153,25 @@ WidgetClass smeBSBObjectClass = (WidgetClass) &smeBSBClassRec;
  ************************************************************/
 
 /*	Function Name: ClassInitialize
- *	Description: Initializes the SmeBSBObject. 
+ *	Description: Initializes the SmeBSBObject class. 
  *	Arguments: none.
  *	Returns: none.
  */
 
-static void 
-ClassInitialize()
+static void ClassInitialize()
 {
+    static XtConvertArgRec screenConvertArg[] = {
+      {XtWidgetBaseOffset, (XtPointer)XtOffsetOf(WidgetRec, core.screen), sizeof(Screen *)},
+      {XtWidgetBaseOffset, (XtPointer)XtOffsetOf(WidgetRec, core.colormap), sizeof(Colormap)},
+    };
+
     XawInitializeWidgetSet();
     XtAddConverter( XtRString, XtRJustify, XmuCvtStringToJustify, NULL, 0 );
+
+    XtAddConverter(XtRString, XtRPixmap, XawCvtStringToPixmap,
+         screenConvertArg, XtNumber(screenConvertArg));
+    XtAddConverter(XtRString, XtRClipMask, XawCvtStringToClipMask,
+         screenConvertArg, XtNumber(screenConvertArg));
 }
 
 /*      Function Name: Initialize
@@ -323,18 +334,17 @@ Cardinal *num_args;
     if (entry->rectangle.sensitive != old_entry->rectangle.sensitive )
 	ret_val = TRUE;
 
-    if (entry->sme_bsb.left_bitmap != old_entry->sme_bsb.left_bitmap) {
+    if ((entry->sme_bsb.left_bitmap != old_entry->sme_bsb.left_bitmap) ||
+	(entry->sme_bsb.left_clipmask != old_entry->sme_bsb.left_clipmask)) {
 	GetBitmapInfo(new, TRUE);
 	ret_val = TRUE;
     }
 
-    if (entry->sme_bsb.right_bitmap != old_entry->sme_bsb.right_bitmap) {
+    if ((entry->sme_bsb.right_bitmap != old_entry->sme_bsb.right_bitmap) ||
+	(entry->sme_bsb.right_clipmask != old_entry->sme_bsb.right_clipmask)) {
 	GetBitmapInfo(new, FALSE);
 	ret_val = TRUE;
     }
-    if ((entry->sme_bsb.left_clipmask != old_entry->sme_bsb.left_clipmask) ||
-        (entry->sme_bsb.right_clipmask != old_entry->sme_bsb.right_clipmask))
-	ret_val = TRUE;
 
     if ((old_entry->sme_bsb.font != entry->sme_bsb.font) ||
 	(old_entry->sme_bsb.foreground != entry->sme_bsb.foreground) ||
