@@ -32,12 +32,15 @@ SOFTWARE.
 #include <X11/XawPlus/Scrollbar.h>
 #include <X11/XawPlus/ViewportP.h>
 
-static void ScrollUpDownProc(), ThumbProc();
-static Boolean GetGeometry();
 
-static void ComputeWithForceBars();
+/****************************************************************
+ *
+ * Predefined resources of Viewport
+ *
+ ****************************************************************/
 
 #define offset(field) XtOffsetOf(ViewportRec, viewport.field)
+
 static XtResource resources[] = {
     {XtNforceBars, XtCBoolean, XtRBoolean, sizeof(Boolean),
 	 offset(forcebars), XtRImmediate, (XtPointer)False},
@@ -54,15 +57,32 @@ static XtResource resources[] = {
 };
 #undef offset
 
-static void Initialize(), ConstraintInitialize(),
-    Realize(), Resize(), ChangeManaged();
+/****************************************************************
+ *
+ * Prototypes
+ *
+ ****************************************************************/
+
+static Boolean GetGeometry();
+static void ComputeWithForceBars();
+static void ScrollUpDownProc(), ThumbProc();
+
+static void Initialize(), ConstraintInitialize(), Realize(), Resize(), ChangeManaged();
 static Boolean SetValues(), Layout();
 static XtGeometryResult GeometryManager(), PreferredGeometry();
 
+
+/****************************************************************
+ *
+ * Class record of Viewport
+ *
+ ****************************************************************/
+
 #define superclass	(&formClassRec)
+
 ViewportClassRec viewportClassRec = {
   { /* core_class fields */
-    /* superclass	  */	(WidgetClass) superclass,
+    /* superclass	  */	(WidgetClass)superclass,
     /* class_name	  */	"Viewport",
     /* widget_size	  */	sizeof(ViewportRec),
     /* class_initialize	  */	XawInitializeWidgetSet,
@@ -119,9 +139,15 @@ ViewportClassRec viewportClassRec = {
   }
 };
 
-
 WidgetClass viewportWidgetClass = (WidgetClass)&viewportClassRec;
 
+
+/****************************************************************
+ *
+ * Private Procedures
+ *
+ ****************************************************************/
+
 static Widget CreateScrollbar(w, horizontal)
     ViewportWidget w;
     Boolean horizontal;
@@ -173,6 +199,12 @@ static Widget CreateScrollbar(w, horizontal)
     return bar;
 }
 
+/****************************************************************
+ *
+ * Methods of this Widget
+ *
+ ****************************************************************/
+
 static void Initialize(request, new, args, n_args)
 Widget request, new;
 ArgList args;
@@ -194,6 +226,11 @@ Cardinal *n_args;
     w->viewport.child = (Widget) NULL;
     w->viewport.horiz_bar = w->viewport.vert_bar = (Widget)NULL;
 
+/*
+ * Install the Action procedures
+ */
+   
+
 /* 
  * Create Clip Widget.
  */
@@ -988,6 +1025,11 @@ static XtGeometryResult PreferredGeometry(w, constraints, reply)
 	return XtGeometryYes;
 }
 
+/************************************************************
+ *
+ *  Public routines. 
+ *
+ ************************************************************/
 
 void
 #if NeedFunctionPrototypes
