@@ -65,6 +65,8 @@ extern LabelClassRec labelClassRec;
 typedef struct {
     /* resources */
     Boolean	resize;
+    Boolean	truncLabel;
+    Boolean	truncLeftSide;
     Pixel	foreground;
     XFontStruct	*font;
     unsigned char encoding;
@@ -73,10 +75,12 @@ typedef struct {
     Dimension	internal_width;
     Dimension	internal_height;
     Pixmap	pixmap;
+    Pixmap      clipMask;
     Pixmap	left_bitmap;
+    Pixmap	left_clipMask;
 
     /* private state */
-    Pixmap	xpm_clipmask;		/* for XPM clipmask support */
+    char	*label_full_len;	/* saved label string if label was truncated */
     GC		normal_GC;
     GC          gray_GC;		/* for displaying insensitive text labels */
     GC		lgray_GC;
